package Chisel
import ChiselError._
import Node._
import scala.collection.mutable.ArrayBuffer

class ROM[T <: Data](val lits: Seq[Literal], gen: () => T) extends Vec[T](gen) {
  val data = gen().toNode
  inferWidth = fixWidth(gen().getWidth)

  override def read(addr: UFix): T = {
    var port = new ROMRead(this, addr)
    val data = gen().asOutput
    data assign port
    data.setIsTypeNode
    data
  }

  override def write(addr: UFix, data: T) = {
    ChiselErrors += ChiselError("Can't write to ROM", Thread.currentThread().getStackTrace)
  }

  override def isReg = true
}

class ROMRead[T <: Data](val rom: ROM[T], addri: Bits) extends Node {
  def addr = inputs(0)
  inputs += addri
  inputs += rom

  inferWidth = fixWidth(rom.data.getWidth)

  override def toString: String = rom + "[" + addr + "]"
}
